# -*- coding: utf-8 -*-
"""
Created on Tue Feb 23 15:23:06 2021

@author: nerea
"""
import math
import matplotlib.pyplot as plt
from mpl_toolkits import mplot3d
import numpy as np
import time
from skyfield.api import load
import Grabitatea3DOrokor as grab


if __name__ == "__main__":

    N = 10
    G = 8.888*math.pow(10, -10)
    eps = math.pow(10, -2)  # Orbitak bira oso bat eman duen bereizteko
    loop = 100000  # Zenbat denbora pausu kalkulatuko diren
    jump = 100  # Zenbat datu grafikatuko diren
    h = 1
    x_all = np.empty((N, int(loop/jump)), dtype=float)
    y_all = np.empty((N, int(loop/jump)), dtype=float)
    z_all = np.empty((N, int(loop/jump)), dtype=float)
    m = np.array([333000, 0.05528, 0.81500, 1.0, 0.10745,
                  317.83, 95.159, 14.536, 17.147, 0.0021])

    ts = load.timescale()
    tnow = ts.now()

    planets = load('de421.bsp')

    sun = planets['sun']
    earth = planets['earth']
    mars = planets['mars']
    mercury = planets['mercury']
    venus = planets['venus']
    jupiter = planets['JUPITER BARYCENTER']
    saturn = planets['SATURN BARYCENTER']
    uranus = planets['URANUS BARYCENTER']
    neptune = planets['NEPTUNE BARYCENTER']
    pluton = planets['PLUTO BARYCENTER']

    sun_pos = sun.at(tnow).position.au
    earth_pos = earth.at(tnow).position.au
    mars_pos = mars.at(tnow).position.au
    mercury_pos = mercury.at(tnow).position.au
    venus_pos = venus.at(tnow).position.au
    jupiter_pos = jupiter.at(tnow).position.au
    saturn_pos = saturn.at(tnow).position.au
    uranus_pos = uranus.at(tnow).position.au
    neptune_pos = neptune.at(tnow).position.au
    pluton_pos = pluton.at(tnow).position.au

    sun_vel = sun.at(tnow).velocity.au_per_d
    earth_vel = earth.at(tnow).velocity.au_per_d
    mars_vel = mars.at(tnow).velocity.au_per_d
    mercury_vel = mercury.at(tnow).velocity.au_per_d
    venus_vel = venus.at(tnow).velocity.au_per_d
    jupiter_vel = jupiter.at(tnow).velocity.au_per_d
    saturn_vel = saturn.at(tnow).velocity.au_per_d
    uranus_vel = uranus.at(tnow).velocity.au_per_d
    neptune_vel = neptune.at(tnow).velocity.au_per_d
    pluton_vel = pluton.at(tnow).velocity.au_per_d

    # Gorputzen posizioak gorde
    x = np.array([sun_pos[0], mercury_pos[0], venus_pos[0], earth_pos[0],
                  mars_pos[0], jupiter_pos[0], saturn_pos[0], uranus_pos[0],
                  neptune_pos[0], pluton_pos[0]])
    y = np.array([sun_pos[1], mercury_pos[1], venus_pos[1], earth_pos[1],
                  mars_pos[1], jupiter_pos[1], saturn_pos[1], uranus_pos[1],
                  neptune_pos[1], pluton_pos[1]])
    z = np.array([sun_pos[2], mercury_pos[2], venus_pos[2], earth_pos[2],
                  mars_pos[2], jupiter_pos[2], saturn_pos[2], uranus_pos[2],
                  neptune_pos[2], pluton_pos[2]])

    x_all[:, 0] = x
    y_all[:, 0] = y
    z_all[:, 0] = z

    # Gorputzen abiadurak gorde
    vx = np.array([sun_vel[0],  mercury_vel[0], venus_vel[0], earth_vel[0],
                   mars_vel[0], jupiter_vel[0], saturn_vel[0], uranus_vel[0],
                   neptune_vel[0], pluton_vel[0]])
    vy = np.array([sun_vel[1],  mercury_vel[1], venus_vel[1], earth_vel[1],
                   mars_vel[1], jupiter_vel[1], saturn_vel[1], uranus_vel[1],
                   neptune_vel[1], pluton_vel[1]])
    vz = np.array([sun_vel[2],  mercury_vel[2], venus_vel[2], earth_vel[2],
                   mars_vel[2], jupiter_vel[2], saturn_vel[2], uranus_vel[2],
                   neptune_vel[2], pluton_vel[2]])

    exet = time.time()

    # t=1/2 aldiuneko abiadura lortzeko
    ax, ay, az = grab.axyz(N, G, 0, m, x, y, z)
    vx = vx + ax*h/2
    vy = vy + ay*h/2
    vz = vz + az*h/2

    # Periodoak gordetzeko
    per = np.full([N], None)

    for t in range(1, loop):
        # Leapfrog algoritmoaren hasieran , x(t=0), v(t=1/2) eta a(t=0) ditugu
        x = x + vx*h
        y = y + vy*h
        z = z + vz*h
        ax, ay, az = grab.axyz(N, G, 0, m, x, y, z)
        vx = vx + ax*h
        vy = vy + ay*h
        vz = vz + az*h

        # Orbita oso bat bada hasierako posiziotik True gorde
        per_bete = np.logical_and(abs(x - x_all[:, 0]) < eps,
                                  abs(y - y_all[:, 0]) < eps,
                                  abs(z - z_all[:, 0]) < eps)
        if t % jump == 0:
            x_all[:, int(t/jump)] = x
            y_all[:, int(t/jump)] = y
            z_all[:, int(t/jump)] = z

        # Periodoa kalkulatzeko (t>10 hasierako posizioak kontuan ez hartzeko)
        for i in range(0, N):
            if per_bete[i] == True and per[i] == None and t > 10:
                per[i] = h*t

    # Lurraren periodoarekin normalizatu
    per = np.around(per.astype(np.double)/per[3], decimals=3)

    # --------------------GRAFIKATZEA----------------------
    axis = plt.axes(projection='3d', facecolor='black')

    axis.plot3D(x_all[0, :], y_all[0, :], z_all[0, :], linewidth=0.01)
    axis.plot3D(x_all[1, :], y_all[1, :], z_all[1, :], linewidth=0.1)
    axis.plot3D(x_all[2, :], y_all[2, :], z_all[2, :], linewidth=0.1)
    axis.plot3D(x_all[3, :], y_all[3, :], z_all[3, :], linewidth=0.1)
    axis.plot3D(x_all[4, :], y_all[4, :], z_all[4, :], linewidth=0.1,
                label='Marte')
    axis.plot3D(x_all[5, :], y_all[5, :], z_all[5, :], linewidth=1.5,
                label='Jupiter')
    axis.plot3D(x_all[6, :], y_all[6, :], z_all[6, :], linewidth=1.5,
                label='Saturno')
    axis.plot3D(x_all[7, :], y_all[7, :], z_all[7, :], linewidth=1.5,
                label='Urano')
    axis.plot3D(x_all[8, :], y_all[8, :], z_all[8, :], linewidth=1.5,
                label='Neptuno')
    axis.plot3D(x_all[9, :], y_all[9, :], z_all[9, :], linewidth=1.5,
                label='Pluton')

    # Simulazioa egin den eguneko planeten posizioak markatu
    axis.scatter(x_all[0, 0], y_all[0, 0], z_all[0, 0], s=0)
    axis.scatter(x_all[1, 0], y_all[1, 0], z_all[1, 0], s=0)
    axis.scatter(x_all[2, 0], y_all[2, 0], z_all[2, 0], s=0)
    axis.scatter(x_all[3, 0], y_all[3, 0], z_all[3, 0], s=0)
    axis.scatter(x_all[4, 0], y_all[4, 0], z_all[4, 0], s=0)
    axis.scatter(x_all[5, 0], y_all[5, 0], z_all[5, 0])
    axis.scatter(x_all[6, 0], y_all[6, 0], z_all[6, 0])
    axis.scatter(x_all[7, 0], y_all[7, 0], z_all[7, 0])
    axis.scatter(x_all[8, 0], y_all[8, 0], z_all[8, 0])
    axis.scatter(x_all[9, 0], y_all[9, 0], z_all[9, 0])

    axis.grid(False)
    axis.set_axis_off()

    plt.legend()
    plt.show()

    # ------------Periodoak pantailan idatzi-----------
    print('Merkurioren periodoa ' + str(per[1]) + ' urte da.')
    print('Artizarraren periodoa ' + str(per[2]) + ' urte da.')
    print('Lurraren periodoa ' + str(per[3]) + ' urte da.')
    print('Marteren periodoa ' + str(per[4]) + ' urte da.')
    print('Jupiterren periodoa ' + str(per[5]) + ' urte da.')
    print('Saturnoren periodoa ' + str(per[6]) + ' urte da.')
    print('Uranoren periodoa ' + str(per[7]) + ' urte da.')
    print('Neptunoren periodoa ' + str(per[8]) + ' urte da.')
    print('Plutonen periodoa ' + str(per[9]) + ' urte da.\n\r')

    print('Exekuzio denbora ' + str(round(time.time() - exet, 2)) +
          's izan da.')
