# -*- coding: utf-8 -*-
"""
Created on Sun Mar 28 17:37:34 2021

@author: nerea

Programa honetan karpeta bat sortuko dugu simulazio ezberdinak gordetzeko.
Karpena honetan bi fitxategi gordeko dira. Batetik, simulazioko datuak
(h,g,loop...), eta azken aldiuneko datuak (xn, v(n-1/2)). 
Beste fitxategi bateak kalkuluak gordeko dira.
Ondoren GIF_N_3D.py programaren bitartez gif-a sortuko da karpeta horretan.
"""

import numpy as np
import time
import os
import Talkak3DOrokor as talk
import Grabitatea3DOrokor as grab


if __name__ == "__main__":
    N = 100
    D = 1.0
    n = 0.1
    G = 1.0
    mtot = 1.0
    sigma = 100
    m = np.full((N), mtot)
    x = np.random.normal(loc=0, scale=sigma, size=N)
    y = np.random.normal(loc=0, scale=sigma, size=N)
    z = np.random.normal(loc=0, scale=sigma, size=N)

    vx = np.random.normal(loc=0.0, scale=0, size=N)
    vy = np.random.normal(loc=0.0, scale=0, size=N)
    vz = np.random.normal(loc=0.0, scale=0, size=N)

    aber = time.time()
    h = 0.2
    loop = 1000
    kont = 1
    karpeta = 0

    while True:
        try:
            os.mkdir('Grabitazio-kolapsoa_Emaitza' + str(karpeta))
            karpetaIzena = 'Grabitazio-kolapsoa_Emaitza' + str(karpeta)
            break
        except OSError:
            karpeta = karpeta + 1

    if os.name == 'posix':
        slash = '/'
    else:
        slash = '\\'

    fitx1 = open(str(karpetaIzena) + str(slash) + 'Informazioa' +
                 str(karpeta) + '.txt', 'w')

    fitx1.write('N = ' + str(N) + '\n')
    fitx1.write('D = ' + str(D) + '\n')
    fitx1.write('n = ' + str(n) + '\n')
    fitx1.write('G = ' + str(G) + '\n')
    fitx1.write('h = ' + str(h) + '\n')
    fitx1.write('loop = ' + str(loop) + '\n')
    fitx1.write('mtot = ' + str(mtot) + '\n')
    fitx1.write('xmax = ' + str(np.amax(x)) + '\n')
    fitx1.write('xmin = ' + str(np.amin(x)) + '\n')
    fitx1.write('ymax = ' + str(np.amax(y)) + '\n')
    fitx1.write('ymin = ' + str(np.amin(y)) + '\n')
    fitx1.write('zmax = ' + str(np.amax(z)) + '\n')
    fitx1.write('zmin = ' + str(np.amin(z)) + '\n')

    fitx1.write('\n----------Hasierako x-------------\n')
    np.savetxt(fitx1, x, delimiter=' ', newline=' ')
    fitx1.write('\n----------Hasierako y-------------\n')
    np.savetxt(fitx1, y, delimiter=' ', newline=' ')
    fitx1.write('\n----------Hasierako z-------------\n')
    np.savetxt(fitx1, z, delimiter=' ', newline=' ')
    fitx1.write('\n----------Hasierako vx-------------\n')
    np.savetxt(fitx1, vx, delimiter=' ', newline=' ')
    fitx1.write('\n----------Hasierako vy-------------\n')
    np.savetxt(fitx1, vy, delimiter=' ', newline=' ')
    fitx1.write('\n----------Hasierako vz-------------\n')
    np.savetxt(fitx1, vz, delimiter=' ', newline=' ')


    fitx2 = open(str(karpetaIzena) + str(slash) + 'Datuak' +
                str(karpeta) + '.txt', 'w')
    np.savetxt(fitx2, x, delimiter=' ', newline=' ')
    np.savetxt(fitx2, y, delimiter=' ', newline=' ')
    np.savetxt(fitx2, z, delimiter=' ', newline=' ')
    fitx2.write('\n')

    Dij = (np.transpose(np.ones([N, N])*D) + np.ones([N, N])*D)/2

    # Hasierako baldintzak ezarri
    vx, vy, vz = talk.talkaEbatzi(N, Dij, n, m, x, y, z, vx, vy, vz)
    ax, ay, az = grab.axyz(N, G, Dij, m, x, y, z)
    vx = vx + ax*h/2
    vy = vy + ay*h/2
    vz = vz + az*h/2

    # Leapfrog algoritmoa
    for t in range(1, loop):
        x = x + vx*h
        y = y + vy*h
        z = z + vz*h
        vx, vy, vz = talk.talkaEbatzi(N, Dij, n, m, x, y, z, vx, vy, vz)
        ax, ay, az = grab.axyz(N, G, Dij, m, x, y, z)
        vx = vx + ax*h
        vy = vy + ay*h
        vz = vz + az*h

        np.savetxt(fitx2, x, delimiter=' ', newline=' ')
        np.savetxt(fitx2, y, delimiter=' ', newline=' ')
        np.savetxt(fitx2, z, delimiter=' ', newline=' ')
        fitx2.write('\n')
        if t % int(loop/5) == 0:
            print(str(loop-t) + "  iterazio geratzen dira.")

    fitx1.write('\nTime = ' + str(time.time() - aber))
    fitx1.write('\n----------Azken x-------------\n')
    np.savetxt(fitx1, x, delimiter=' ', newline=' ')
    fitx1.write('\n----------Azken y-------------\n')
    np.savetxt(fitx1, y, delimiter=' ', newline=' ')
    fitx1.write('\n----------Azken z-------------\n')
    np.savetxt(fitx1, z, delimiter=' ', newline=' ')
    fitx1.write('\n----------Azken vx-------------\n')
    np.savetxt(fitx1, vx, delimiter=' ', newline=' ')
    fitx1.write('\n----------Azken vy-------------\n')
    np.savetxt(fitx1, vy, delimiter=' ', newline=' ')
    fitx1.write('\n----------Azken vz-------------\n')
    np.savetxt(fitx1, vz, delimiter=' ', newline=' ')
    fitx1.close()
    fitx2.close()
    print('Programa honek ' + str(time.time() - aber) + 's behar izan' +
          'ditu exekutatzeko')