# -*- coding: utf-8 -*-
"""
Created on Fri May 14 18:54:34 2021

@author: nerea
"""

import numpy as np


class Kutxa():

    def __init__(self, x, y, z, aldea):
        """
            Kutxa klaseko objektu bat hasieratzeko aldagaiak:

            x: kutxaren zentroaren posizioaren x koordenatua (float)
            y: kutxaren zentroaren posizioaren y koordenatua (float)
            z: kutxaren zentroaren posizioaren z koordenatua (float)
            aldea: kutxaren aldea (float)

        """

        self.zentroa = (x, y, z)
        self.aldea = aldea

    def txokatu(self, x, y, z):
        """
            Partikulek kutxako pareten kontra jasandako talkak detektatzen
            dituen metodoa.

        Jaso:
            x: partikulen x koordenatuko posizioak (N tamainako np array-a,
                                                    elementuak float)
            y: partikulen y koordenatuko posizioak (N tamainako np array-a,
                                                    elementuak float)
            z: partikulen z koordenatuko posizioak (N tamainako np array-a,
                                                    elementuak float)
        Itzuli:
            txokea: partikula batek pareten kontra talkarik jasaten badu True
                    itzuli (N tamainako np array-a, elementuak float)
        """

        txokeax = np.where(np.logical_and(x < self.zentroa[0] + self.aldea/2.0,
                           x > self.zentroa[0] - self.aldea/2.0),
                           True, False)
        txokeay = np.where(np.logical_and(y < self.zentroa[1] + self.aldea/2.0,
                           y > self.zentroa[1] - self.aldea/2.0),
                           True, False)
        txokeaz = np.where(np.logical_and(z < self.zentroa[2] + self.aldea/2.0,
                           z > self.zentroa[2] - self.aldea/2.0),
                           True, False)
        txokea = np.logical_not(txokeax*txokeay*txokeaz)
        return txokea

    def KutxaTxokea(self, n, x, y, z, vx, vy, vz):
        """
            Partikulek kutxako pareten kontra talka egitean abiadurak nola
            aldatzen diren kalkulatzen duen metodoa.

        Jaso:
            n: talka-parametroa (float, [0,1] tarteko balio)
            x: partikulen x koordenatuko posizioak (N tamainako np array-n,
                                                    elementuak float)
            y: partikulen y koordenatuko posizioak (N tamainako np array-n,
                                                    elementuak float)
            z: partikulen z koordenatuko posizioak (N tamainako np array-n,
                                                    elementuak float)
            vx: partikulen x koordenatuko abiadurak (N tamainako np array-n,
                                                    elementuak float)
            vy: partikulen y koordenatuko abiadurak (N tamainako np array-n,
                                                    elementuak float)
            vz: partikulen z koordenatuko abiadurak (N tamainako np array-n,
                                                    elementuak float)

        Itzuli:
            vx: partikulen x koordenatuko abiadura berriak (N tamainako np
                                                            array-n, elementuak
                                                            float)
            vy: partikulen y koordenatuko abiadura berriak (N tamainako np
                                                            array-n, elementuak
                                                            float)
            vz: partikulen z koordenatuko abiadura berriak (N tamainako np
                                                            array-n, elementuak
                                                            float)
        """
        talka = self.txokatu(x, y, z)
        result = np.where(talka == True)
        try:
            for i in np.nditer(result):
                if x[i] > self.zentroa[0] + self.aldea/2.0:
                    vx[i] = -n*vx[i]
                elif x[i] < self.zentroa[0] - self.aldea/2.0:
                    vx[i] = -n*vx[i]
                if y[i] > self.zentroa[1] + self.aldea/2.0:
                    vy[i] = -n*vy[i]
                elif y[i] < self.zentroa[1] - self.aldea/2.0:
                    vy[i] = -n*vy[i]
                if z[i] > self.zentroa[2] + self.aldea/2.0:
                    vz[i] = -n*vz[i]
                elif z[i] < self.zentroa[2] - self.aldea/2.0:
                    vz[i] = -n*vz[i]
        except:
            pass
        return vx, vy, vz
