# -*- coding: utf-8 -*-
"""
Created on Tue Apr 13 15:06:44 2021

@author: nerea
"""

import numpy as np
import random
import math

def talkaExis(N, Dij, x, y, z):
    """
       Talka jasango duten partikula bikoteak detektatu eta modu aleatorio
       batean partikula bikoteak ordenatzen dituen funtzioa.

    Jaso:
        N: partikula kopurua (int)
        Dij: partikulen erradioen batura (N X N tamainako np array-n,
                                          elementuak float)
        x: partikulen x koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        y: partikulen y koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        z: partikulen z koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)

    Itzuli:
        listOfCoordinates:  tuplaz osatutako lista, tuplako osagaiak talka-
                            bikotean parte hartzen duten partikulen indizeak
                            dira: [(part1, part2), (part2, part5),....]

    """

    # Diagonaleko elementuen arteko talka eta talka bakoitza bi aldiz
    # ez kalkulatzeko
    eps = math.pow(10, -2)
    Rlagun = dist(N, x, y, z) + (Dij + eps)*np.tril(np.ones([N, N]))

    # talka gertatuko bada True gorde.
    talka = Rlagun <= Dij

    # bere baitan True parametroa duten elementuen indizeak gorde tuplatan
    result = np.where(talka == True)
    listOfCoordinates = list(zip(result[0], result[1]))
    # tuplak ausaz berrordenatu
    random.shuffle(listOfCoordinates)

    return listOfCoordinates


def dist(N, x, y, z):
    """
        Partikulen arteko distantzia kalkulatzeko funtzioa.

    Jaso:
        N: partikula kopurua (int)
        x: partikulen x koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        y: partikulen y koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        z: partikulen z koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)

    Itzuli:
        R: partikulen arteko distantzia (NXN tamainako np array-n,
                                                    elementuak float)

    """

    xij = np.transpose(np.ones([N, N])*x) - np.ones([N, N])*x
    yij = np.transpose(np.ones([N, N])*y) - np.ones([N, N])*y
    zij = np.transpose(np.ones([N, N])*z) - np.ones([N, N])*z
    R = np.power(np.power(xij, 2) + np.power(yij, 2) + np.power(zij, 2), 0.5)
    return R


def talkaExekutatu(N, n, listOfCoor, m, x, y, z, vx, vy, vz):
    """
       Talka jasango duten partikula-bikoteen lista exekutatuko duen
       funtzioa hiru dimentsiotan.

    Jaso:
        N: partikula kopurua (int)
        n: talka-parametroa (float, [0,1] tarteko balio)
        listOfCoor: tuplaz osatutako lista, tuplako osagaiak talka-
                    bikotean parte hartzen duten partikulen indizeak
                    dira: [(part1, part2), (part3, part5),....]
        m: partikulen masa (N tamainako np array-n, elementuak float)
        x: partikulen x koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        y: partikulen y koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        z: partikulen z koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        vx: partikulen x koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)
        vy: partikulen y koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)
        vz: partikulen z koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)

    Itzuli:
        vx: partikulen x koordenatuko abiadura berriak (N tamainako np array-n,
                                                        elementuak float)
        vy: partikulen y koordenatuko abiadura berriak (N tamainako np array-n,
                                                        elementuak float)
        vz: partikulen z koordenatuko abiadura berriak (N tamainako np array-n,
                                                        elementuak float)

    """


    # i iterazio bakoitzean talka bat exekutatuko da
    for i in range(0, len(listOfCoor)):
        m1 = m[listOfCoor[i][0]]
        m2 = m[listOfCoor[i][1]]
        matrix = np.array(([m1, m2, 0, 0, 0, 0], [0, 0, m1, m2, 0, 0],
                           [0, 0, 0, 0, m1, m2], [1, -1, 0, 0, 0, 0],
                           [0, 0, 1, -1, 0, 0], [0, 0, 0, 0, 1, -1]),
                            dtype=float)

        # vx, vy eta vz behin baino gehiagotan indexatzea ekiditeko
        vx1 = vx[listOfCoor[i][0]]
        vx2 = vx[listOfCoor[i][1]]
        vy1 = vy[listOfCoor[i][0]]
        vy2 = vy[listOfCoor[i][1]]
        vz1 = vz[listOfCoor[i][0]]
        vz2 = vz[listOfCoor[i][1]]

        # Koordenatu bakoitzeko partikulen arteko distantzia kalkulatu
        rx = x[listOfCoor[i][0]] - x[listOfCoor[i][1]]
        ry = y[listOfCoor[i][0]] - y[listOfCoor[i][1]]
        rz = z[listOfCoor[i][0]] - z[listOfCoor[i][1]]

        # Talka ondorengo abiadura erlatiboak kalkulatu
        vxij, vyij, vzij = vErlatibo3D(n, rx, ry, rz, vx1 - vx2, vy1 - vy2,
                                       vz1-vz2)

        # Partikula bakoitzaren talka ondorengo abiadura kalkulatu
        b = np.array(([m1*vx1 + m2*vx2, m1*vy1 + m2*vy2, m1*vz1 + m2*vz2, vxij,
                       vyij, vzij]))
        sol = np.linalg.solve(matrix, b)

        # Talkako emaitzak gorde
        vx[listOfCoor[i][0]] = sol[0]
        vx[listOfCoor[i][1]] = sol[1]
        vy[listOfCoor[i][0]] = sol[2]
        vy[listOfCoor[i][1]] = sol[3]
        vz[listOfCoor[i][0]] = sol[4]
        vz[listOfCoor[i][1]] = sol[5]

    return vx, vy, vz


def vErlatibo3D(n, rx, ry, rz, vx, vy, vz):
    """
       Bi partikulen arteko talka ondorengo abiadura erlatiboa kalkulatzen
       duen funtzioa hiru dimentsiotan.

    Jaso:
        n: talka-parametroa (float, [0,1] tarteko balio)
        rx: partikulen arteko x koordenatuko distantzia (float)
        ry: partikulen arteko y koordenatuko distantzia (float)
        rz: partikulen arteko z koordenatuko distantzia (float)
        vx: partikulen arteko x koordenatuko abiadura talka aurretik (float)
        vy: partikulen arteko y koordenatuko abiadura talka aurretik (float)
        vz: partikulen arteko z koordenatuko abiadura talka aurretik (float)

    Itzuli:
        vem[0]: partikulen arteko x koordenatuko abiadura talka ondoren (float)
        vem[1]: partikulen arteko y koordenatuko abiadura talka ondoren (float)
        vem[2]: partikulen arteko z koordenatuko abiadura talka ondoren (float)

    """

    # Normalizazio faktoreak
    rnor = math.pow(rx*rx + ry*ry + rz*rz, -0.5)
    tnor1 = math.pow(math.pow(rx + rz, 2) + 2*ry*ry, -0.5)
    tnor2 = math.pow(math.pow((rz*rx + ry*ry + rz*rz), 2) +
                     math.pow(rz*rx + ry*ry + rx*rx, 2) +
                     math.pow((rx - rz)*ry, 2), -0.5)

    # Abiadura normala eta tangentziala kalkulatu
    P = np.array(([rx*rnor, ry*rnor, rz*rnor], [tnor1*ry, -tnor1*(rx + rz),
                  tnor1*ry], [-tnor2*(rz*rx + ry*ry + rz*rz),
                  tnor2*(rx - rz)*ry, tnor2*(rz*rx + ry*ry + rx*rx)]))
    vem = np.array([vx, vy, vz])
    vkor = np.matmul(P, vem)

    # Osagai normala talka ondoren aldatu
    vkor[0] = -n*vkor[0]
    vem = np.matmul(P.T, vkor)

    return vem[0], vem[1], vem[2]


def talkaEbatzi(N, Dij, n, m, x, y, z, vx, vy, vz):
    """
        Partikulen arteko talka detektatu eta exekutatzen duen funtzioa
        hiru dimentsiotan.

    Jaso:
        N: partikula kopurua (int)
        Dij: partikulen erradioen batura (N X N tamainako np array-n,
                                          elementuak float)
        n: talka-parametroa (float, [0,1] tarteko balio)
        m: partikulen masa (N tamainako np array-n, elementuak float)
        x: partikulen x koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        y: partikulen y koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        z: partikulen z koordenatuko posizioak (N tamainako np array-n,
                                                elementuak float)
        vx: partikulen x koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)
        vy: partikulen y koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)
        vz: partikulen z koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)

    Itzuli:
        vx: partikulen x koordenatuko abiadura berriak (N tamainako np array-n,
                                                        elementuak float)
        vy: partikulen y koordenatuko abiadura berriak (N tamainako np array-n,
                                                        elementuak float)
        vz: partikulen z koordenatuko abiadura berriak (N tamainako np array-n,
                                                        elementuak float)
    """
    listOfCoor = talkaExis(N, Dij, x, y, z)
    vx, vy, vz = talkaExekutatu(N, n, listOfCoor, m, x, y, z, vx, vy, vz)
    return vx, vy, vz


def energiaZinetiko(m, vx, vy, vz):
    """
        Partikulen energia zinetiko totala kalkulatu.

    Jaso:
        m: partikulen masa (N tamainako np array-n, elementuak float)
        vx: partikulen x koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)
        vy: partikulen y koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)
        vz: partikulen z koordenatuko abiadurak (N tamainako np array-n,
                                                elementuak float)

    Itzuli:
        np.sum(Ezi): Partikula bakoitzaren energia zitenikoaren batura (float)

    """
    Ezi = m*(vx*vx + vy*vy + vz*vz)/2
    return np.sum(Ezi)

if __name__ == "__main__":
    N = 2
    D = 3.0
    n = 0
    m = np.array([1, 1], dtype=float)
    x = np.array([-1, 1], dtype=float)
    y = np.array([0, 0], dtype=float)
    z = np.array([0, 0], dtype=float)
    vx = np.array([1, -2], dtype=float)
    vy = np.array([0, 0], dtype=float)
    vz = np.array([0, 0], dtype=float)
    print("Hasieran :")
    print("vx = " + str(vx))
    print("vy = " + str(vy))
    print("vz = " + str(vz))
    print(np.sum(vx*vx))
    Dij = (np.transpose(np.ones([N, N])*D) + np.ones([N, N])*D)/2
    vx, vy, vz = talkaEbatzi(N, Dij, n, m, x, y, z, vx, vy, vz)
    print("Talka eta gero :")
    print("vx = " + str(vx))
    print("vy = " + str(vy))
    print("vz = " + str(vz))
    print(np.sum(vx*vx))